------------------------------------------------------------------------------------------
-- HEIG-VD ///////////////////////////////////////////////////////////////////////////////
-- Haute Ecole d'Ingenerie et de Gestion du Canton de Vaud
-- School of Business and Engineering in Canton de Vaud
------------------------------------------------------------------------------------------
-- REDS Institute ////////////////////////////////////////////////////////////////////////
-- Reconfigurable Embedded Digital Systems
------------------------------------------------------------------------------------------
--
-- File                 : DE1_SOC_golden_top.vhd
-- Author               : Flavio Capitao (FCC)
-- Date                 : 13.04.2017
--
-- Context              : spi_buttons_leds project
--
------------------------------------------------------------------------------------------
-- Description : DE1-SoC golden top
--
--               SW9   : choose master ('1',UP) or slave ('0',DOWN)
--               LEDR9 : effective master ('1',ON) or slave ('0',OFF)
--               LEDR8 : slave select line state and
--                       valid configuration (only one master request)
--
--               SW[7:0] and LEDR[7:0] : values exchange by SPI
--
------------------------------------------------------------------------------------------
-- Dependencies : -
--
------------------------------------------------------------------------------------------
-- Modifications :
-- Ver    Date        Engineer      Comments
-- 0.0    See header  FCC           Initial version from DE1_SOC_golden_top.v (Altera)
--
------------------------------------------------------------------------------------------

library ieee ;
  use ieee.std_logic_1164.all ;
  use ieee.numeric_std.all ;

entity DE1_SOC_golden_top is
  port (
    CLOCK_50    : in    std_logic;
    GPIO_0      : inout std_logic_vector(35 downto 0);
    GPIO_1      : inout std_logic_vector(35 downto 0);
    HEX0        : out   std_logic_vector(6 downto 0);
    HEX1        : out   std_logic_vector(6 downto 0);
    HEX2        : out   std_logic_vector(6 downto 0);
    HEX3        : out   std_logic_vector(6 downto 0);
    HEX4        : out   std_logic_vector(6 downto 0);
    HEX5        : out   std_logic_vector(6 downto 0);
    KEY         : in    std_logic_vector(3 downto 0);
    LEDR        : out   std_logic_vector(9 downto 0);
    SW          : in    std_logic_vector(9 downto 0)
    --VGA_B       : out   std_logic_vector(7 downto 0);
    --VGA_BLANK_N : out   std_logic;
    --VGA_CLK     : out   std_logic;
    --VGA_G       : out   std_logic_vector(7 downto 0);
    --VGA_HS      : out   std_logic;
    --VGA_R       : out   std_logic_vector(7 downto 0);
    --VGA_SYNC_N  : out   std_logic;
    --VGA_VS      : out   std_logic
  ) ;
end entity ; -- DE1_SOC_golden_top

architecture rtl of DE1_SOC_golden_top is

  -- SPI I/O wrapping
  alias sclk_line_a           : std_logic is GPIO_1(0); -- pull-up
  alias mosi_line_a           : std_logic is GPIO_1(1); -- pull-up
  alias miso_line_a           : std_logic is GPIO_1(2); -- pull-up
  alias n_ss_line_a           : std_logic is GPIO_1(3); -- pull-up
  alias master_request_line_a : std_logic is GPIO_1(4); -- pull-up

  -- Control/status I/O wrapping
  alias master_request_a : std_logic is SW(9);
  alias master_led_a     : std_logic is LEDR(9);
  alias ss_led_a         : std_logic is LEDR(8);

  -- User I/O wrapping
  alias user_n_rst_a   : std_logic is KEY(3);
  alias user_keys_a    : std_logic_vector(2 downto 0) is KEY(2 downto 0);
  alias user_switchs_a : std_logic_vector(7 downto 0) is SW(7 downto 0);
  alias user_leds_a    : std_logic_vector(7 downto 0) is LEDR(7 downto 0);

  -- SPI user signals
  signal master_en_s : std_logic;
  signal mst_sclk_s  : std_logic;
  signal mst_mosi_s  : std_logic;
  signal mst_miso_s  : std_logic;
  signal mst_n_ss_s  : std_logic;
  signal slv_sclk_s  : std_logic;
  signal slv_mosi_s  : std_logic;
  signal slv_miso_s  : std_logic;
  signal slv_n_ss_s  : std_logic;

  -- user signals
  signal user_rst_s : std_logic;
begin

  -- not used
  GPIO_0              <= (others => 'Z');
  GPIO_1(35 downto 4) <= (others => 'Z');
  HEX0                <= (others => '1');
  HEX1                <= (others => '1');
  HEX2                <= (others => '1');
  HEX3                <= (others => '1');
  HEX4                <= (others => '1');
  HEX5                <= (others => '1');

  ----------------------
  --       SPI
  ----------------------

  -- master_request_line_a is a wired AND between the FPGAs master_request_a
  -- if master_request_line_a='1' both FPGAs are asking to be master
  master_request_line_a <= 'Z' when master_request_a='1' else '0'; -- pull-up

  -- we are master only if we ask for it and the other FPGA don't
  master_en_s  <= '1' when (master_request_a='1' AND master_request_line_a='0') else '0';
  master_led_a <= master_en_s;

  -- SCLK line
  sclk_line_a <= '0' when (mst_sclk_s='0'  AND master_en_s='1') else 'Z';
  slv_sclk_s  <= '0' when (sclk_line_a='0' AND master_en_s='0') else '1';

  -- MOSI line
  mosi_line_a <= '0' when (mst_mosi_s='0'  AND master_en_s='1') else 'Z';
  slv_mosi_s  <= '0' when (mosi_line_a='0' AND master_en_s='0') else '1';

  -- MISO line
  miso_line_a <= '0' when (slv_miso_s='0'  AND master_en_s='0') else 'Z';
  mst_miso_s  <= '0' when (miso_line_a='0' AND master_en_s='1') else '1';

  -- nSS line
  n_ss_line_a <= '0' when (mst_n_ss_s='0'  AND master_en_s='1') else 'Z';
  slv_n_ss_s  <= '0' when (n_ss_line_a='0' AND master_en_s='0') else '1';
  ss_led_a    <= not slv_n_ss_s;


  ----------------------
  --       User
  ----------------------
   user_rst_s <= not user_n_rst_a;

  user_top : entity work.spi_buttons_leds_top
  port map(
    clk_i       => CLOCK_50,
    rst_i       => user_rst_s,
    keys_i      => user_keys_a,
    switchs_i   => user_switchs_a,
    leds_o      => user_leds_a,
    master_en_i => master_en_s,
    sclk_o      => mst_sclk_s,
    mosi_o      => mst_mosi_s,
    miso_i      => mst_miso_s,
    n_ss_o      => mst_n_ss_s,
    sclk_i      => slv_sclk_s,
    mosi_i      => slv_mosi_s,
    miso_o      => slv_miso_s,
    n_ss_i      => slv_n_ss_s
  );


end architecture ; -- rtl
