------------------------------------------------------------------------------------------
-- HEIG-VD ///////////////////////////////////////////////////////////////////////////////
-- Haute Ecole d'Ingenerie et de Gestion du Canton de Vaud
-- School of Business and Engineering in Canton de Vaud
------------------------------------------------------------------------------------------
-- REDS Institute ////////////////////////////////////////////////////////////////////////
-- Reconfigurable Embedded Digital Systems
------------------------------------------------------------------------------------------
--
-- File                 : spi_buttons_leds_top.vhd
-- Author               : Flavio Capitao (FCC)
-- Date                 : 12.04.2017
--
-- Context              : spi_buttons_leds project
--
------------------------------------------------------------------------------------------
-- Description : Exchange buttons and LEDs states between 2 cards through SPI
--
--               SPI :
--                      - Slave select (n_ss) is active low
--
------------------------------------------------------------------------------------------
-- Dependencies : -
--
------------------------------------------------------------------------------------------
-- Modifications :
-- Ver    Date        Engineer      Comments
-- 0.0    See header  FCC           Initial version.
--
------------------------------------------------------------------------------------------

use work.log_pkg.all;

library ieee ;
    use ieee.std_logic_1164.all ;
    use ieee.numeric_std.all ;

entity spi_buttons_leds_top is
  port (
    clk_i       : in  std_logic; -- 50Mhz
    rst_i       : in  std_logic; -- KEY3
    keys_i      : in  std_logic_vector(2 downto 0);
    switchs_i   : in  std_logic_vector(7 downto 0);
    leds_o      : out std_logic_vector(7 downto 0);
    master_en_i : in  std_logic;
    -- Master interface
    sclk_o      : out std_logic;
    mosi_o      : out std_logic;
    miso_i      : in  std_logic;
    n_ss_o      : out std_logic;
    -- Slave interface
    sclk_i      : in  std_logic;
    mosi_i      : in  std_logic;
    miso_o      : out std_logic;
    n_ss_i      : in  std_logic
  );
end entity ; -- spi_buttons_leds_top

architecture rtl of spi_buttons_leds_top is

    -- SPI
    constant SPI_LENGTH      : integer := 8;
    constant CLK_DIV         : integer := 50;-- freq SCLK = 50MHz/50 = 1Mhz

    signal master_start_s    : std_logic;
    signal master_end_s      : std_logic;
    signal master_data_in_s  : std_logic_vector(SPI_LENGTH-1 downto 0);
    signal master_data_out_s : std_logic_vector(SPI_LENGTH-1 downto 0);

    signal slave_load_s      : std_logic;
    signal slave_end_s       : std_logic;
    signal slave_data_in_s   : std_logic_vector(SPI_LENGTH-1 downto 0);
    signal slave_data_out_s  : std_logic_vector(SPI_LENGTH-1 downto 0);

    -- Timer
    constant TIMER_US        : integer := 200000; -- us
    constant TIMER_COUNT     : integer := 50*TIMER_US; -- iterations=freq*timerTime
    signal   timer_s         : std_logic;
    signal   timer_counter_s : unsigned(ilogup(TIMER_COUNT-1) downto 0);
begin

    ----------------------
    --       SPI
    ----------------------

    spi_master : entity work.spi_master
    generic map (
        N       => SPI_LENGTH,
        CLK_DIV => CLK_DIV
    )
    port map (
        clk_i           => clk_i,
        rst_i           => rst_i,
        tx_start_i      => master_start_s,
        tx_end_o        => master_end_s,
        data_parallel_i => master_data_in_s,
        data_parallel_o => master_data_out_s,
        sclk_o          => sclk_o,
        n_ss_o          => n_ss_o,
        mosi_o          => mosi_o,
        miso_i          => miso_i
    );

    spi_slave : entity work.spi_slave
    generic map (
        N => SPI_LENGTH
    )
    port map (
        clk_i           => clk_i,
        rst_i           => rst_i,
        tx_load_i       => slave_load_s,
        tx_end_o        => slave_end_s,
        data_parallel_i => slave_data_in_s,
        data_parallel_o => slave_data_out_s,
        sclk_i          => sclk_i,
        n_ss_i          => n_ss_i,
        mosi_i          => mosi_i,
        miso_o          => miso_o
    );

    master_start_s <= timer_s;
    --master_end_s;
    master_data_in_s <= switchs_i;

    slave_load_s   <= timer_s;
    --slave_end_s;
    slave_data_in_s <= switchs_i;

    leds_o <= master_data_out_s when master_en_i='1' else slave_data_out_s;


    ----------------------
    --       Timer
    ----------------------
    -- generate timer pulse that will start the spi transfert every 200ms
    gen_timer_tick : process( clk_i )
    begin
        if rising_edge(clk_i) then
            if rst_i = '1' then--OR master_en_i='0' then
                timer_counter_s <= to_unsigned(TIMER_COUNT-1,timer_counter_s'length);
                timer_s   <= '0';
            else
                timer_counter_s <= timer_counter_s-1;
                timer_s   <= '0';
                if timer_counter_s=0 then
                    timer_counter_s <= to_unsigned(TIMER_COUNT-1,timer_counter_s'length);
                    timer_s         <= '1';
                end if ;
            end if ;
        end if ;
    end process ; -- gen_timer_tick

end architecture ; -- rtl
